@@ -16,6 +16,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "manette-mapping-error.h"
+#include "manette-mapping-error-private.h"
 
 G_DEFINE_QUARK (manette-mapping-error, manette_mapping_error)
