@@ -16,7 +16,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "manette-mapping-manager.h"
+#include "manette-mapping-manager-private.h"
 
 #include <glib.h>
 #include <glib-object.h>
@@ -45,7 +45,7 @@ static guint signals[N_SIGNALS];
 
 #define CONFIG_DIR "libmanette"
 #define MAPPING_CONFIG_FILE "gamecontrollerdb"
-#define MAPPING_RESOURCE_URI "resource:///org/gnome/aplazas/libmanette/gamecontrollerdb"
+#define MAPPING_RESOURCE_URI "resource:///org/gnome/Manette/gamecontrollerdb"
 
 /* Private */
 
